% This function set calculates KP and two-step test sizes and then generates Table 1 panel C. 
% It contains several functions that is used in the main function Func_table1c.m
function [res]=Func_table1c(T,tau,dmu,dbeta,ddelta,errnum,testnum,cv,cvone,option,mrep)

retwo_5 = zeros(mrep,1);  reone = zeros(mrep,1);
parfor m = 1:mrep

    rng(1*mrep+m, 'twister');
    [y, x] = DGP_c_table1(T,tau,dmu,dbeta,ddelta,errnum);
    [~, k] = size(x); R = [1, -1];
    [sup_wald_one] = wald_withI0(y,x,R,testnum,option);
    reone(m) = (cvone <= sup_wald_one);     
	retwo_5(m) = twostep_SB_withI0(y,x,cv,cvone,testnum,option);

end
res = [sum(reone); sum(retwo_5)]/mrep;

end

%%% --- Below are some functions built in the main function Func_table1c.m, some of these are the same in Func_table1ab.m ---
% This function constructs the KP wald statistic with I(0) variables
function [sup_wald, br_ssr]=wald_withI0(y,x,R,testnum,option)

T = length(y);  %%% R = [1, -1];
beta = (x'*x)\(x'*y); u0 = y-x*beta; ssr0 = u0'*u0;
dimVhat = length(R(1,:)); num = 2*length(testnum)-1;
t1=round(T*.15); t2=round(T*.85); ssrdiff=zeros(t2-t1+1,1);
delta_all = zeros(t2-t1+1,num+1); Vhat_all=zeros(t2-t1+1,dimVhat,dimVhat); j=t1;
while j<=t2
    [x1, xtest, xrest] = transx(x,j,testnum);
    b1 = (x1'*x1)\(x1'*y); u1 = y-x1*b1;  ssr1 = u1'*u1;
    delta_all(j-t1+1,:) = b1(end-num:end); regs = M(xrest)*xtest;
    ssrdiff(j-t1+1,:)=(ssr0-ssr1);  dimregs = length(regs(1,:));
    KT = LRV_I0(u0,u1,regs);
    Vhat_all(j-t1+1,:,:) = (T-dimregs)*inv(xtest'*regs)*KT*(T-dimregs)*inv(xtest'*regs);
    j=j+1;
end

[~, maxssrdiff_loc] = max(ssrdiff);
br_ssr = maxssrdiff_loc+t1-1;

if length(testnum) == 1 pqb=1;pqf=1; else pqb=2;pqf=0; end
if option == 1   % option 1 estimate all Vhat
    
    Ft=zeros(t2-t1+1,1); j=t1;
    while j<=t2
        delta = delta_all(j-t1+1,:)';
        Vhat = squeeze(Vhat_all(j-t1+1,:,:));
        Ft(j-t1+1,:) = T*delta'*R'*inv(R*Vhat*R')*R*delta;
        j=j+1;
    end
    [sup_wald, maxf_loc] = max(Ft);
    br_f = maxf_loc+t1-1;
    
elseif option == 2  % option 2 estimate under optimal break point Vhat
    
    Vhat = squeeze(Vhat_all(maxssrdiff_loc,:,:));
    Ft=zeros(t2-t1+1,1); j=t1;
    while j<=t2
        delta = delta_all(j-t1+1,:)';
        Ft(j-t1+1,:) = T*delta'*R'*inv(R*Vhat*R')*R*delta;
        j=j+1;
    end
    [sup_wald, maxf_loc] = max(Ft);
    br_f = maxf_loc+t1-1;
    
elseif option == 3  % option 3 estimate under br_ssr 
    
    Vhat = squeeze(Vhat_all(maxssrdiff_loc,:,:));
    delta = delta_all(maxssrdiff_loc,:)';
    sup_wald = T*delta'*R'*inv(R*Vhat*R')*R*delta;
    br_f = br_ssr;    
end

end

% This function conducts the two-step test with I(0) variables, result=1 means rejection, result=0 means non-rejection.
function res = twostep_SB_withI0(y,x,cv,cvone,testnum,option)


[T, k] = size(x); 
R = [eye(3), -eye(3)];
[sup_wald, br] = wald_withI0(y,x,R,[1:k],option);
rej1 = (cv <= sup_wald); rej2 = 0;
if rej1 == 1
    xnew = transx(x,br,setdiff(1:k,testnum));
    beta = (xnew'*xnew)\(xnew'*y); u0 = y-xnew*beta; 
    [xnew2, xtest2, xrest2] = transx(xnew,br,1:length(testnum));
    beta2 = (xnew2'*xnew2)\(xnew2'*y); u2 = y-xnew2*beta2; 
    	
	delta = beta2(end-1:end); regs = M(xrest2)*xtest2;
	KT = LRV_I0(u0,u2,regs);	R2 = [1, -1]; dimregs = length(regs(1,:));
	Vhat = (T-dimregs)*inv(xtest2'*regs)*KT*(T-dimregs)*inv(xtest2'*regs);
	F = T*delta'*R2'*inv(R2*Vhat*R2')*R2*delta;
    rej2 = (chi2inv(0.95,length(testnum)) <= F);
    %% 5% default significance, other levels are subject to change
end

res = rej1*rej2;
end

% This function constructs the reconstructed matrix given break date used in various scenarios.
function [xnew, xtest, xrest] = transx(x,t,testno)

[T,p] = size(x); len = length(testno);
xtest = [[x(1:t,testno); zeros(T-t,len)], [zeros(t,len); x(t+1:T,testno)]];
xrest = x(:,setdiff(1:p,testno));
xnew = [xrest, xtest];

end

% This function constructs the projection matrix M.
function mx = M(x)

if isempty(x) 
    mx=1; 
else
    mx=eye(length(x(:,1)))-x*inv(x'*x)*x';
end
end